/*
punycode.c from RFC 3492
http://www.nicemice.net/idn/
Adam M. Costello
http://www.nicemice.net/amc/


This is ANSI C code (C89) implementing Punycode (RFC 3492).

*/

/************************************************************/
/* Public interface (would normally go in its own .h file): */
#ifndef _INCLUDED_PUNYCODE_HEADER_H_
#define _INCLUDED_PUNYCODE_HEADER_H_

#include <limits.h>
#ifdef _WIN32
#include <tchar.h>		//+++
#endif

enum punycode_status {
  punycode_success,
  punycode_bad_input,	/* Input is invalid.					   */
  punycode_big_output,	/* Output would exceed the space provided. */
  punycode_overflow 	/* Input needs wider integers to process.  */
};

#if UINT_MAX >= (1 << 26) - 1
typedef unsigned int punycode_uint;
#else
typedef unsigned long punycode_uint;
#endif

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/
enum punycode_status punycode_encode(
  punycode_uint input_length,
  const punycode_uint input[],
  const unsigned char case_flags[],
  punycode_uint *output_length,
  char output[] );
	/* punycode_encode() converts Unicode to Punycode.	The input	  */
	/* is represented as an array of Unicode code points (not code	  */
	/* units; surrogate pairs are not allowed), and the output		  */
	/* will be represented as an array of ASCII code points.  The	  */
	/* output string is *not* null-terminated; it will contain		  */
	/* zeros if and only if the input contains zeros.  (Of course	  */
	/* the caller can leave room for a terminator and add one if	  */
	/* needed.)  The input_length is the number of code points in	  */
	/* the input.  The output_length is an in/out argument: the 	  */
	/* caller passes in the maximum number of code points that it	  */
	/* can receive, and on successful return it will contain the	  */
	/* number of code points actually output.  The case_flags array   */
	/* holds input_length boolean values, where nonzero suggests that */
	/* the corresponding Unicode character be forced to uppercase	  */
	/* after being decoded (if possible), and zero suggests that	  */
	/* it be forced to lowercase (if possible).  ASCII code points	  */
	/* are encoded literally, except that ASCII letters are forced	  */
	/* to uppercase or lowercase according to the corresponding 	  */
	/* uppercase flags.  If case_flags is a null pointer then ASCII   */
	/* letters are left as they are, and other code points are		  */
	/* treated as if their uppercase flags were zero.  The return	  */
	/* value can be any of the punycode_status values defined above   */
	/* except punycode_bad_input; if not punycode_success, then 	  */
	/* output_size and output might contain garbage.				  */
	/* punycode_encode()jR[hvjR[hɕϊ܂B͂  */
	/* jR[hR[h|Cg̔ziR[hPʁG㗝΂͋   */
	/* jƂĕ\܂Aďo͂͂`rbhhR[h|Cg */
	/* zƂĕ\ł傤Bo͕ȃkŏIā */
	/* 񁖁F́A͂[܂łꍇɌAĂ */
	/* ̏ꍇɂ͕KA[܂łł傤BiĂяol */
	/* ͕KvȂE𖾎邽߂ɒǉXy[XcƂł */
	/* ܂Bjinput_length͓̓R[h|Cg̐łBoutput_length*/
	/* ͓o͈łFĂяol͎󂯎邱ƂłR[h|C */
	/* g̍ő̐nAĖ߂ꍇAۂɏo͂ꂽR[h */
	/* |Cg̐܂łł傤Bcase_flagszinput_length */
	/* u[lA[ȊOłꍇ͑Ή郆jR[hA */
	/* i\łȂjAǂꂽő啶ɋ邱Ƃ */
	/* Ă܂Aă[Ai\łȂjɋ */
	/* 邱Ƃ܂B`rbhhR[h|CgA`rbhh */
	/* ΉuppercasetOɏ]đ啶邢͏ɂ*/
	/* ȊOÂ܂܃R[h܂Bcase_flagsk|C^ */
	/* ȂA`rbhĥ܂܂ŁÃR[h|Cg	  */
	/* uppercasetO[ł邩̂悤Ɉ܂BAl	  */
	/* punycode_bad_inputȊO͏Œ`punycode_statusl̂ǂ  */
	/* łGpunycode_successłȂ΁Aoutput_sizeƏo͖͂Ӗ  */
	/* Ȓl܂ł邩܂B 							  */

enum punycode_status punycode_decode(
  punycode_uint input_length,
 #ifdef _WIN32
  const TCHAR	input[],
 #else
  const char	input[],
 #endif
  punycode_uint *output_length,
  punycode_uint output[],
  unsigned char case_flags[] );

#ifdef __cplusplus
}
#endif /*__cplusplus */
	/* punycode_decode() converts Punycode to Unicode.	The input is  */
	/* represented as an array of ASCII code points, and the output   */
	/* will be represented as an array of Unicode code points.	The   */
	/* input_length is the number of code points in the input.	The   */
	/* output_length is an in/out argument: the caller passes in	  */
	/* the maximum number of code points that it can receive, and	  */
	/* on successful return it will contain the actual number of	  */
	/* code points output.	The case_flags array needs room for at	  */
	/* least output_length values, or it can be a null pointer if the */
	/* case information is not needed.	A nonzero flag suggests that  */
	/* the corresponding Unicode character be forced to uppercase	  */
	/* by the caller (if possible), while zero suggests that it be	  */
	/* forced to lowercase (if possible).  ASCII code points are	  */
	/* output already in the proper case, but their flags will be set */
	/* appropriately so that applying the flags would be harmless.	  */
	/* The return value can be any of the punycode_status values	  */
	/* defined above; if not punycode_success, then output_length,	  */
	/* output, and case_flags might contain garbage.  On success, the */
	/* decoder will never need to write an output_length greater than */
	/* input_length, because of how the encoding is defined.		  */
	/* punycode_decode()vjR[hjR[hɕϊ܂B͂  */
	/* `rbhhR[h|CgzƂĕ\Ao͂̓jR[h   */
	/* R[h|CgzƂĕ\ł傤Binput_length͓ */
	/* R[h|Cg̐łBoutput_lengtho͈łFĂяo  */
	/* l͂ꂪ󂯎邱ƂłR[h|Cg̍ő̐nA */
	/* ċAꍇ͏o͂ꂽR[h|Cg̎ۂ̐܂ */
	/* ł傤Bcase_flagsźAȂƂoutput_lengthl̋  */
	/* ԂKvA邢͑啶񂪕KvȂȂk|C^ */
	/* ł蓾܂BnonzerotOΉ郆jR[hAĂяo  */
	/* lɂāi\łȂj啶ɉt邱Ƃ */
	/* A[i\łƂĂjɋ */
	/* ƂƂ܂B`rbhhR[h|CgK؂ȑ啶 */
	/* ŏo͂܂ÃtÓAtOp */
	/* ƂQłł悤ɁAK؂ɐݒ肳ł傤B߂l */
	/* ͏ɒ`punycode_statusl̂ǂꂩłG			  */
	/* punycode_successłȂ΁Aoutput_lengthƏo͂case_flags  */
	/* ӖȒl܂ł邩܂BAR[fBO */
	/* `@ɂAǎ҂͌input_length傫   */
	/* output_lengthKvȂł傤B						  */
#endif/*_INCLUDED_PUNYCODE_HEADER_H_*/
